/**
* The contents of this file are subject to the Mozilla Public 
* License Version 1.1 (the "License"); you may not use this 
* file except in compliance with the License. You may obtain a
* copy of the License at http://www.mozilla.org/MPL/
*
* Software distributed under the License is distributed on an 
* "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express 
* or implied. See the License for the specific language 
* governing rights and limitations under the License.
*
* The Original Code is this file as it was released on
* March 17, 2005.
*
* The Initial Developer of the Original Code is Terry Yuen. 
* Portions created by Terry Yuen are Copyright (C) 2005.
* All Rights Reserved.
**/

/**
 * Dependencies:
 *  /tonguedialect.js::TongueDialect.inWhitelist()
 *  /tonguedialect.js::TongueDialect.addToWhitelist()
 */

function TongueSpeaker(key, wait, done, dict, select) {
  this.key = key;
  this.showWait = wait;
  this.showDone = done;
  this.showDict = dict;
  this.selectedOnly = select;
  this.brand = true;
}

/**
 * Call this everytime you want to activate the tongue.
 */
TongueSpeaker.prototype.init = function(event, menu, form) {
  this.xmlreq = null;
  this.data = null;

  this.menu = menu || event.originalTarget;

  this.formInput = form || this.getFormField(event);  //can be overridden

  if(this.formInput) {
    this.data = this.extractInput();

    this.setWaitStatus();

    if(this.data && !this.inWhitelist(this.data.value)) {
      this.getSuggestions();
    }
    else {
      this.setDoneStatus();
    }
  }
};

/**
 * This can be overridden to support other input controls.
 */
TongueSpeaker.prototype.getFormField = function(event) {
  var input = document.popupNode;
  if(input instanceof HTMLInputElement || input instanceof HTMLTextAreaElement) {

    input = new XPCNativeWrapper(input, "selectionStart", "selectionEnd", "value", "setSelectionRange()", "getAttribute()", "QueryInterface()");

    if(input.getAttribute("type") == "password")return null;

    return input;
  }
  return null;
};

TongueSpeaker.prototype.destroy = function() {
  if(this.xmlreq)this.xmlreq.abort();
  this.cleanUpMenus();
  this.xmlreq = null;
  this.menu = null;
  this.formInput = null;
  this.data = null;
};

TongueSpeaker.prototype.cleanUpMenus = function() {
  while(this.menu && this.menu.hasChildNodes() && this.menu.lastChild.hasAttribute("tongue")) {
    this.menu.removeChild(this.menu.lastChild);
  }
};

TongueSpeaker.prototype.extractInput = function() {
  if(this.formInput.selectionStart < this.formInput.selectionEnd) {
    //quick performance check
    if(this.formInput.selectionEnd - this.formInput.selectionStart > 50)return null;

    //we want to maintain the exact position so we can replace it
    var data = {};
    data.start = this.formInput.selectionStart;
    data.end = this.formInput.selectionEnd;
    data.value = this.formInput.value.substring(data.start, data.end);

    //abort if URL
    if(/^(http|ftp|file)s?:\/\//.test(data.value))return null;

    data.value = this.formInput.value.substring(data.start, data.end);
    return data;

  } else if(!this.selectedOnly && this.formInput.selectionStart != -1) {

    //if the caret is at position zero, check entire string
    if(this.formInput.selectionStart == 0) {

      var data = {};
      data.start = 0;
      data.value = this.formInput.value;
      data.end = data.value.length;

      //quick performance check
      if(data.end == 0 || data.end > 50)return null;

      //abort if URL
      if(/^(http|ftp|file)s?:\/\//.test(data.value))return null;

      return data;

    } else {  //gather text under caret

      var data = {};
      data.start = this.formInput.selectionStart;
      data.end = this.formInput.selectionEnd;

      var len = this.formInput.value.length;
      while(data.start > 0 && this.isChar(this.formInput.value.charAt(data.start - 1)))data.start--;
      while(data.end < len && this.isChar(this.formInput.value.charAt(data.end)))data.end++;

      //make sure at least 1 letter exists
      if(data.start == data.end)return null;

      data.value = this.formInput.value.substring(data.start, data.end);
      return data;
    }

  } else return null;
};

TongueSpeaker.prototype.isChar = function(s) {
  return !((s < "a" || s > "z") && (s < "A" || s > "Z") &&
            (s < "0" || s > "9") && (s != "-"));
};

TongueSpeaker.prototype.setWaitStatus = function() {
  if(this.showWait) {
    this.setMessage(document.getElementById("bundle_riteoftongue").getString("menu.wait"));
  }
};

TongueSpeaker.prototype.setDoneStatus = function() {
  if(this.showDone) {
    this.setMessage(document.getElementById("bundle_riteoftongue").getString("menu.done"));
  } else {
    this.clearMessage();
  }
};

TongueSpeaker.prototype.setMessage = function(str, tip) {
  var item = document.getElementById("riteoftongue-msg");
  if(item) {
    item.setAttribute("label", str);
    if(tip)item.setAttribute("tooltiptext", tip);
    else if(item.hasAttribute("tooltiptext"))item.removeAttribute("tooltiptext");
  } else {
    if(this.menu.hasChildNodes()) {
      item = document.createElement("menuseparator");
      item.setAttribute("tongue", "status");
      this.menu.appendChild(item);
    }
    item = document.createElement("menuitem");
    item.setAttribute("id", "riteoftongue-msg");
    item.setAttribute("label", str);
    if(tip)item.setAttribute("tooltiptext", tip);
    else if(item.hasAttribute("tooltiptext"))item.removeAttribute("tooltiptext");
    item.setAttribute("disabled", "true");
    item.setAttribute("tongue", "status");
    this.menu.appendChild(item);
  }
};

TongueSpeaker.prototype.clearMessage = function() {
  var item = document.getElementById("riteoftongue-msg");
  if(item) {
    if(item.previousSibling && item.previousSibling.hasAttribute("tongue")) {
      item.parentNode.removeChild(item.previousSibling);
    }
    item.parentNode.removeChild(item);
  }
};

TongueSpeaker.prototype.appendMenuItems = function(arr) {
  if(arr.length > 0) {
    this.clearMessage();

    var item;
    if(this.menu.hasChildNodes()) {
      item = document.createElement("menuseparator");
      item.setAttribute("tongue", "status");
      this.menu.appendChild(item);
    }

    const obj = this;  //for inline function pointers
    for(var i=0; i<arr.length; i++) {
      item = document.createElement("menuitem");
      item.setAttribute("class", "riteoftongue");
      item.setAttribute("label", arr[i]);
      item.setAttribute("tongue", "suggestion");
      this.menu.appendChild(item);
      item.addEventListener("command", function(event) {
        obj.itemClick(event);
      }, false);
    }

    if(this.showDict) {
      item = document.createElement("menuitem");
      item.setAttribute("label", document.getElementById("bundle_riteoftongue").getFormattedString("menu.dictionary", [this.data.value]));
      item.setAttribute("tongue", "add");
      this.menu.appendChild(item);
      item.addEventListener("command", function(event) {
        obj.itemClick(event);
      }, false);
    }

    if(this.brand) {
      item = document.createElement("menuitem");
      item.setAttribute("class", "riteoftongue");
      item.setAttribute("label", document.getElementById("bundle_riteoftongue").getString("yahoo"));
      item.setAttribute("disabled", "true");
      item.setAttribute("tongue", "yahoo");
      this.menu.appendChild(item);
    }

  } else this.setDoneStatus();
};

/**
 * This and the callback can be overridden to support other transport protocols (eg., soap).
 */
TongueSpeaker.prototype.getSuggestions = function() {
  this.xmlreq = new XMLHttpRequest();
  this.xmlreq.open("GET", "http://api.search.yahoo.com/WebSearchService/V1/spellingSuggestion?appid=" + this.key + "&query=" + encodeURIComponent(this.data.value), true);
  const obj = this;
  this.xmlreq.onreadystatechange = function() {
    obj.getSuggestionsCallback();
  };
  this.xmlreq.send(null);
};

TongueSpeaker.prototype.getSuggestionsCallback = function() {
  try {
   if(this.xmlreq.readyState == 4) {
      if(this.xmlreq.responseXML) {
        var error = this.getXMLErrors(this.xmlreq.responseXML);
        if(!error) {
          var arr = this.getXMLData(this.xmlreq.responseXML);
          this.appendMenuItems(arr);
        } else {
          this.setMessage(this.xmlreq.status + " " + this.xmlreq.statusText, error);
        }
      } else {
        dump("\nRiteOfTongue: Request not OK. HTTP code was " + this.xmlreq.status);
        this.setMessage(this.xmlreq.status + " " + this.xmlreq.statusText);
      }
    }
  } catch(e) {} //destroy() called part way. End gracefully.
};

TongueSpeaker.prototype.getXMLErrors = function(xml) {
  var error = xml.getElementsByTagName("Message");
  return (error && error.length > 0) ? error[0].firstChild.nodeValue : null;
};

TongueSpeaker.prototype.getXMLData = function(xml) {
  var arr = new Array();
  var results = xml.getElementsByTagName("Result");
  if(results && results.length > 0) {
    arr.push(results[0].firstChild.nodeValue.replace("&lt;","<").replace("&gt;",">").replace("&amp;","&"));
  }
  return arr;
};

TongueSpeaker.prototype.itemClick = function(event) {
  var target = event.originalTarget;
  var type = target.getAttribute("tongue");

  switch(type) {
    case "suggestion":
      //save scroll position
      var doc=null, docTop=0, docLeft=0;
      if("nsIDOMNSEditableElement" in Components.interfaces) {  //Firefox 1.5+
        doc = this.formInput.QueryInterface(Components.interfaces.nsIDOMNSEditableElement).editor.rootElement;
        docTop = doc.scrollTop;
        docLeft = doc.scrollLeft;
      }

      //recover the text
      var newStr = target.getAttribute("label");
      this.formInput.value = this.formInput.value.substring(0, this.data.start) + newStr + this.formInput.value.substring(this.data.end, this.formInput.value.length);
      var tmp = this.data.start + newStr.length;
      this.formInput.setSelectionRange(tmp, tmp);  //position caret at end

      //restore scroll position
      if(doc) {  //Firefox 1.5+
        doc.scrollTop = docTop;
        doc.scrollLeft = docLeft;
      }
      break;

    case "add":
      this.addToWhitelist(this.data.value);
  }
};

TongueSpeaker.prototype.inWhitelist = function(str) {
  return TongueDialect.inWhitelist(str);
};

TongueSpeaker.prototype.addToWhitelist = function(str) {
  TongueDialect.addToWhitelist(str);
};

///////////// Google Implementation /////////////////////////////////////

function TongueSpeaker2(key, wait, done, dict, select) {
  this.key = key;
  this.showWait = wait;
  this.showDone = done;
  this.showDict = dict;
  this.selectedOnly = select;
  this.brand = false;
}

TongueSpeaker2.prototype = new TongueSpeaker();

TongueSpeaker2.prototype.getSuggestions = function() {
  var soap = new SOAPCall();
  soap.transportURI = "http://api.google.com/search/beta2";

  var param = new Array(2);
  param[0] = new SOAPParameter(this.key, "key");
  param[1] = new SOAPParameter(this.data.value, "phrase");
  soap.encode(0, "doSpellingSuggestion", "urn:GoogleSearch", 0, null, param.length, param);

  const obj = this;
  this.xmlreq = soap.asyncInvoke(function(rs, call, err) {
    obj.getSuggestionsCallback(rs, call, err);
  });
};

TongueSpeaker2.prototype.getSuggestionsCallback = function(rs, call, err) {
  if(err == 0) {
    if(!rs.fault) {
      var arr = this.getXMLData(rs);
      this.appendMenuItems(arr);
    } else {
      dump("\nRiteOfTongue: Request not OK. Server returned: " + rs.fault.faultString);
      this.setMessage(rs.fault.faultCode + " Error", rs.fault.faultString);
    }
  } else {
    dump("\nRiteOfTongue: Request not OK. SOAP error code was " + err);
    this.setMessage("SOAP " + err);
  }
};

TongueSpeaker2.prototype.getXMLData = function(rs) {
  var arr = new Array();
  var results = rs.body.getElementsByTagName("return")[0];
  if(results.hasChildNodes()) {
    arr.push(results.firstChild.nodeValue.replace("&lt;","<").replace("&gt;",">").replace("&amp;","&"));
  }
  return arr;
};

/* (tree walker)
var spell_s = "+++++++++++++++++++++++++++++";
function spellit(n, xml) {
  var str = "\n" + spell_s.substring(0,n) + xml.tagName+":"+xml.nodeName+":"+xml.nodeValue;

  if(xml.hasChildNodes()) {
    for(var i=0; i<xml.childNodes.length; i++) {
      str += spellit(n+1, xml.childNodes[i]);
    }
  }
  return str;
}*/